@@ -17,6 +17,8 @@ import sys
 import subprocess
 import shlex
 import shutil
+import random
+import string
 import re
 import json
 import click
@@ -70,6 +72,9 @@ class TgWrap():
     def __init__(self, verbose):
         self.printer = Printer(verbose)
 
+        self.run_id = self.generate_random_string()
+        self.printer.verbose(f"Run id: {self.run_id}")
+
         # Check if the "TERRAGRUNT_SOURCE" environment variable is set
         env_var = "TERRAGRUNT_SOURCE"
         if env_var in os.environ:
@@ -85,6 +90,9 @@ class TgWrap():
         """ Checks if a program is installed on the system """
         return shutil.which(program) is not None
 
+    def generate_random_string(self, length=8):
+        return ''.join(random.choices(string.ascii_lowercase + string.digits, k=length))
+
     def prepare_groups(self, graph, exclude_external_dependencies):
         """ Prepare the list of groups that will be executed """
         groups = []
@@ -115,7 +123,7 @@ class TgWrap():
         """ Gets the directed graph of terragrunt dependencies, and parse it into a graph object """
         graph = None
         try:
-            temp_file = f'{os.environ["TMPDIR"]}tg-dependencies.gv'
+            temp_file = f'{os.environ["TMPDIR"]}tg-dependencies-{self.run_id}.gv'
             with open(temp_file, 'w', encoding='utf8') as f:
                 working_dir_stmt = f'--terragrunt-working-dir {working_dir}' if working_dir else ''
                 command = \
@@ -183,7 +191,7 @@ class TgWrap():
                             if collect_output_file:
                                 collect_output_file.write(f'{directory}{self.SEPARATOR}')
                                 collect_output_file.flush()
-                            temp_file = f'{os.environ["TMPDIR"]}tgwrap-error'
+                            temp_file = f'{os.environ["TMPDIR"]}tgwrap-error-{self.run_id}'
                             messages = ""
                             with open(temp_file, 'w', encoding='utf8') as f:
                                 rc = {'returncode': 0}
@@ -310,7 +318,7 @@ class TgWrap():
             command=command,
             debug=debug,
             exclude_external_dependencies=True if step_by_step else exclude_external_dependencies,
-            non_interactive=False if command in ['apply', 'destroy'] else True,
+            non_interactive=step_by_step if command in ['apply', 'destroy'] else True,
             no_lock=no_lock,
             update_source=update_source,
             no_auto_approve=False,
@@ -320,7 +328,7 @@ class TgWrap():
 
         if step_by_step:
             self.printer.verbose(
-                'This command will be executed in each individual project directory!'
+                f'This command will be executed in each individual project directory:\n$ {cmd}'
                 )
             self.run_di_graph(
                 command=cmd,
@@ -394,8 +402,8 @@ class TgWrap():
             )
 
         # then run it and capture the output
-        temp_file = f'{os.environ["TMPDIR"]}tgwrap-show'
-        json_file = f'{os.environ["TMPDIR"]}planfile.json'
+        temp_file = f'{os.environ["TMPDIR"]}tgwrap-show-{self.run_id}'
+        json_file = f'{os.environ["TMPDIR"]}planfile-{self.run_id}.json'
         with open(temp_file, 'w', encoding='utf8') as f:
             self.run_di_graph(
                 command=cmd,
@@ -473,49 +481,37 @@ class TgWrap():
         if not ts_validation_successful:
             raise Exception("Terrasafe validation failed on one or more projects")
 
-    def promote(self, dry_run, clean_up, working_dir, source_stage, target_stage):
-        """ Promotes the terragrunt config files from one stage to another """
-
-        # do we have a working dir?
-        working_dir = working_dir if working_dir else os.getcwd()
-        source_path = os.path.join(working_dir, source_stage, '')
-        target_path = os.path.join(working_dir, target_stage, '')
-
-        # Check if source exists
-        if not self.is_installed('rsync'):
-            self.printer.error("'rsync' seems not installed. Cannot continue")
-        elif not os.path.exists(source_path):
-            self.printer.error(f"Could not find stage {source_stage} in path {working_dir}. Cannot continue.")
-        else:
-            self.printer.verbose(f"Copying config: {source_path} => {target_path}")
-
-            dry_run_stmt = '--dry-run' if dry_run else ''
-            clean_stmt   = '--delete' if clean_up else ''
-        
-            cmd = f"rsync -aim {dry_run_stmt} {clean_stmt}" + \
-                "--include='terragrunt.hcl' --include='.terraform.lock.hcl' " + \
-                "--exclude='env.hcl' --exclude='.terragrunt-cache/' " + \
-                "--exclude='terragrunt-debug.tfvars.json' --exclude=planfile " + \
-                f"{source_path} {target_path}"
-
-            cmd = re.sub(' +', ' ', cmd)
-
-            rc = subprocess.run(shlex.split(cmd))
-            self.printer.verbose(rc)
-
-    def deploy(self, dry_run, clean_up, working_dir, source_domain, source_stage, target_domain):
-        """ Deploys the terragrunt config files from one domain/state to another """
+    def deploy(self, dry_run, clean_up, include_lock_file, working_dir, source_stage, target_stage, source_domain="", target_domain=""):
+        """ Deploys the terragrunt config files from one stage to another (and possibly to a different domain) """
     
+        if target_domain and not source_domain:
+            raise Exception("Providing a target domain while omitting the source domain is not supported!")
+        if source_domain and not target_domain:
+            raise Exception("Providing a source domain while omitting the target domain is not supported!")
+
+        if target_domain and not target_stage:
+            self.printer.verbose(f"No target stage given, assume the same as source stage")
+            target_stage=source_stage
+
         # do we have a working dir?
         working_dir = working_dir if working_dir else os.getcwd()
+        # the domains will be ignored when omitted as input
         source_path = os.path.join(working_dir, source_domain, source_stage, '')
-        target_path = os.path.join(working_dir, target_domain, source_stage, '')
+        target_path = os.path.join(working_dir, target_domain, target_stage, '')
 
         # Check if source exists
         if not self.is_installed('rsync'):
             self.printer.error("'rsync' seems not installed. Cannot continue")
         elif not os.path.exists(source_path):
             self.printer.error(f"Could not find stage {source_stage} in path {working_dir}. Cannot continue.")
+            if source_domain:
+                self.printer.error(
+                    "Please ensure you are in the directory where your DLZs are located, or use --working-dir option"
+                )
+            else:
+                self.printer.error(
+                    "Please ensure you are in the root of your DLZ, or use --working-dir option"
+                )
         else:
             self.printer.verbose(f"Copying config: {source_path} => {target_path}")
 
@@ -527,14 +523,25 @@ class TgWrap():
 
             dry_run_stmt = '--dry-run' if dry_run else ''
             clean_stmt   = '--delete' if clean_up else ''
+            env_file_stmt   = "--exclude='env.hcl" if source_stage != target_stage else "--include='env.hcl"
+            lock_file_stmt  = "--include='.terraform.lock.hcl'" if include_lock_file \
+                else "--exclude='.terraform.lock.hcl'"
 
             cmd = f"rsync -aim {dry_run_stmt} {clean_stmt} " + \
-                "--include='terragrunt.hcl' --include='.terraform.lock.hcl' " + \
-                "--exclude='env.hcl' --exclude='.terragrunt-cache/' " + \
+                f"--include='terragrunt.hcl' {lock_file_stmt} {env_file_stmt} " + \
+                "--exclude='.terragrunt-cache/' --exclude='.terraform/' " + \
                 "--exclude='terragrunt-debug.tfvars.json' --exclude=planfile " + \
                 f"{source_path} {target_path}"
 
             cmd = re.sub(' +', ' ', cmd)
 
+            self.printer.normal("Will be deploying:")
+            self.printer.normal(f"from: {source_path}")
+            self.printer.normal(f"to:   {target_path}")
+            self.printer.verbose(f"\nUsing command:\n$ {cmd}")
+            response = input("\nDo you want to continue? (y/n) ")
+            if response.lower() != "y":
+                sys.exit(1)
+
             rc = subprocess.run(shlex.split(cmd))
             self.printer.verbose(rc)
